
public class JNISamples
{
	private native String stringAccess(String sStr);
	private native int [] intArrAccess(int [] aiInp);
	private native int stringArrAccess(String [] asInp);
	private native void nativeMethod(int iDepth);
	private synchronized native void syncNativeMethod(Object oSync);
	private native void fieldAccess();
	private native void nativeCatchThrow() throws IllegalArgumentException;

	static
	{
		System.loadLibrary("JNISamples");
	}

	static int iStaticIntFld;
	String sStringFld;

	public static void main(String [] args)
	{
		JNISamples sample = new JNISamples();

		System.out.println("String access test");
		sample.doString();
		System.out.println("int array test");
		sample.doIntArray();
		System.out.println("String array test");
		sample.doStrArray();
		System.out.println("Callback test");
		sample.callback(0);
		System.out.println("Field access test");
		sample.doFldAccess();
		System.out.println("Native exception catch and throw test");
		try
		{
			sample.nativeCatchThrow();
		}
		catch(IllegalArgumentException iae)
		{
			iae.printStackTrace();
		}
		System.out.println("Synchronized Native Methods");
		sample.syncNativeMethod(new Object());
		System.out.println("Success!");
	}

	private void javaCatchThrow() throws NullPointerException
	{
		throw new NullPointerException("Null pointer exception thrown in javaCatchThrow");
	}

	private void doFldAccess()
	{
		iStaticIntFld = 100;
		sStringFld = "sStringFld";

		fieldAccess();

		System.out.println("In Java");
		System.out.println("iStaticIntFld = " + iStaticIntFld);
		System.out.println("sStringFld = " + sStringFld);
	}

	private void doString()
	{
		String sRet = stringAccess("String test");
		System.out.println(sRet);
	}

	private void doStrArray()
	{
		String [] asInp = new String[10];
		int iIndex, iLen;

		for(iIndex=0; iIndex<10; iIndex++)
		{
			asInp[iIndex] = "String: " + Integer.toString(iIndex);
		}

		iLen = stringArrAccess(asInp);

		System.out.println("String array size: " + iLen);
	}

	private void doIntArray()
	{
		int [] aiInp = new int[10];
		int iIndex;

		for(iIndex=0; iIndex<10; iIndex++)
		{
			aiInp[iIndex] = iIndex;
		}

		int [] aiOut = intArrAccess(aiInp);

		System.out.print("Int array: {");
		for(iIndex=0; iIndex<10; iIndex++)
		{
			System.out.print(aiInp[iIndex] + ", ");
		}
		System.out.println("}");
	}

	private void callback(int iDepth)
	{
		System.out.println("In Java callback. Depth=" + iDepth);
		if(5 > iDepth)
		{
			nativeMethod(iDepth + 1);
		}
		else
		{
			System.out.println("Stopping recursion");
		}
	}
}

